require 'net/http'
require 'cgi'
require 'uri'

class CacheUrl
	attr_accessor :datePath
	attr_accessor :cachePath
	attr_reader :responseCode
	
	def initialize(datePath = nil,cachePath = nil)
		@datePath = datePath
		@cachePath = cachePath
	end
	
	def get(url)
	
		uri = URI.parse(url)
		
		reqserver = uri.host
		reqpath = uri.path 
		if uri.query != nil
			reqpath += "?" + uri.query
		end if
		Net::HTTP.version_1_2

		http = Net::HTTP.new(reqserver, uri.port)
		req = Net::HTTP::Get.new(reqpath)
		
		url = reqserver +  reqpath
		if (updatedate = getUrlLogDate(url)) && (@datePath != nil)
			req["If-Modified-Since"] = updatedate
		end
		
		res = http.request(req)
		responseCode = res.code
		case res.code
		when "200"
			getbody = res.body

			setUrlCache(url,getbody)
			setUrlLogDate(url,res["Last-Modified"])
		when "304"
			getbody = getUrlCache(url)
		else
			getbody = ""
		end
		
		return {"contents" => getbody,"code" => res.code}

	end 

	def getUrlLogDate(url)
		filename = @datePath + "/" + CGI.escape(url)
		begin 
			file = open(filename,"r")
			logdate = file.gets 
			file.close
			return logdate
		rescue
			return
		end	
	end
	private :getUrlLogDate

	def setUrlLogDate(url,moddate)
		
		if moddate == nil
			return
		end
		
		filename = @datePath + "/" + CGI.escape(url)
		begin 
			file = open(filename,"w")
			logdate = file.write(moddate) 
			file.close
		rescue
			return
		end	
	end
	private :setUrlLogDate
	
	def getUrlCache(url)
		filename = @cachePath + "/" + CGI.escape(url)
		begin 
			file = open(filename,"r")
			logdate = file.read
			file.close
			return logdate
		rescue
			return
		end	
	end
	private :getUrlCache
	
	def setUrlCache(url,data)
		filename = @cachePath + "/" + CGI.escape(url)
		begin 
			file = open(filename,"w")
			logdate = file.write(data) 
			file.close
		rescue
			return
		end	
	end
	private :setUrlCache
	
end

